/*
 * stsh.c
 *
 * systrace shell
 *
 * Copyright (c) 2002 Dug Song <dugsong@monkey.org>
 */

#include <sys/param.h>
#include <sys/sysctl.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define PATH_SYSTRACE	"/bin/systrace"

static struct shell_path {
	char		*name;
	char		*path;
} shell_paths[] = {
	{ "sksh",	"/bin/ksh" },
	{ "scsh",	"/bin/csh" },
	{ "szsh",	"/usr/local/bin/zsh" },
	{ "sbash",	"/usr/local/bin/bash" },
	{ "stcsh",	"/usr/local/bin/tcsh" },
	{ NULL }
};

int
main(int argc, char *argv[])
{
	extern char *__progname;
	struct shell_path *sp;
	struct kinfo_proc kp;
	char **args, *p, buf[BUFSIZ];
	int i, j, mib[4];

	/* Determine which shell we're being invoked as. */
	p = __progname;
	if (*p == '-')
		p++;
	
	for (sp = shell_paths; sp->name != NULL; sp++) {
		if (strcmp(p, sp->name) == 0)
			break;
	}
	if (sp->name == NULL)
		errx(1, "invalid shell: %s", p);
	
	/* Determine if we're already being systraced. */
	mib[0] = CTL_KERN;
	mib[1] = KERN_PROC;
	mib[2] = KERN_PROC_PID;
	mib[3] = getpid();

	i = sizeof(kp);
	
	if (sysctl(mib, 4, &kp, &i, NULL, 0) < 0)
		err(1, "KERN_PROC_PID");

	if ((args = malloc(sizeof(*args) * (argc + 8))) == NULL)
		err(1, "malloc");
	
	j = 0;

	/* Make sure we're always systraced. */
	if ((kp.kp_proc.p_flag & P_SYSTRACE) == 0) {
		args[j++] = PATH_SYSTRACE;
		args[j++] = "-a";
#if 0
		args[j++] = "-d";
		args[j++] = "/some/other/policydir";
#else
		args[j++] = "-U";
#endif
		/* Set our shell environment. */
		snprintf(buf, sizeof(buf), "SHELL=%s", sp->path);
		putenv(buf);
	}
	args[j++] = sp->path;
	
	/* If invoked as -shell, it's a login shell. */
	if (*argv[0] == '-')
		args[j++] = "-l";

	/* Copy over and terminate original arguments. */
	for (i = 1; i < argc; i++)
		args[j++] = argv[i];
	
	args[j] = NULL;
	
	execv(args[0], args);
	
	/* NOTREACHED */
	err(1, "exec");
}
